#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/5 16:20
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : version.py.py
from __future__ import annotations

__version__ = '0.0.1'