#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 12:23
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : liyi_tokenizer.py
from __future__ import annotations

from liyi_cute.processor.tokenizers.tokenizer import Tokenizer


class LiyiTokenizer(Tokenizer):
    def __init__(self):
        pass