#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/4/2 12:13
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : jieba_tokenizer.py

from __future__ import annotations

from liyi_cute.processor.tokenizers.tokenizer import Tokenizer


class JiebaTokenizer(Tokenizer):
    pass