#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 10:50
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : constants.py

from __future__ import annotations


GRADIENTS = ["linear-gradient(#FF2D2D, #ff7575, #FF9797)",
             "linear-gradient(#FF95CA, #ffaad5, #FFC1E0)",
             "linear-gradient(#FF8EFF, #ffa6ff, #FFBFFF)",
             "linear-gradient(#CA8EFF, #d3a4ff, #DCB5FF)",
             "linear-gradient(#AAAAFF, #B9B9FF, #CECEFF)",
             "linear-gradient(#84C1FF, #97CBFF, #ACD6FF)",
             "linear-gradient(#4DFFFF, #80FFFF, #A6FFFF)",
             "linear-gradient(#4EFEB3, #7AFEC6, #96FED1)",
             "linear-gradient(#79FF79, #93FF93, #A6FFA6)",
             "linear-gradient(#CCFF80, #D3FF93, #DEFFAC)",
             "linear-gradient(#FFFF93, #FFFFAA, #FFFFB9)",
             "linear-gradient(#FFE66F, #FFED97, #FFFOAC)",
             "linear-gradient(#FFC78E, #FFD1A4, #FFDCB9)",
             "linear-gradient(#FF9D6F, #FFAD86, #FFBD9D)",
             "linear-gradient(#C48888, #CF9E9E, #D9B3B3)",
             "linear-gradient(#B9B973, #C2C287, #CDCD9A)",
             "linear-gradient(#81C0C0, #95CACA, #A3D1D1)",
             "linear-gradient(#A6A6D2, #B8B8DC, #C7C7E2)",
             "linear-gradient(#C07AB8, #CA8EC2, #D2A2CC)",
             ]



































