#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/3/27 17:39
# @Author  : Yizheng Dai
# @Email   : 387942239@qq.com
# @File    : liyi_example_data.py
from __future__ import annotations

from .readerwriter import ExamplesDataReader, ExamplseDataWriter, JsonExamplesDataReader

class LiyiWriter(ExamplesDataReader):
    pass
